/*!
 * Copyright (C) 2001-2005 by egnite Software GmbH. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY EGNITE SOFTWARE GMBH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EGNITE
 * SOFTWARE GMBH OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.ethernut.de/
 */

/*!
 * $Log$
 */

/* Required for strlen(). */
#include <string.h>

/* Contains stdio prototypes like fopen(), printf() etc. */
#include <stdio.h>

/* Required for _ioctl(). */
#include <io.h>

/* Hardware specific defaults like DEV_DEBUG and DEV_UART1. */
#include <dev/board.h>

/* Required for NutVersionString(). */
#include <sys/version.h>

/* Required for NutSleep(). */
#include <sys/timer.h>

/*
 * RS485 Slave Example. 
 */
int main(void)
{
    /* Baudrate for debug output. */
    u_long baud_dbg = 115200;
    /* RS485 baudrate. */
    u_long baud_485 = 1200;
    /* RS485 flow control. */
    u_long flow_485 = USART_MF_HALFDUPLEX;
    /* RS485 receive time out in milliseconds. */
    u_long to_485 = 2000;
    /* RS485 stream. */
    FILE *rs485;
    /*
     * Actually we should never ever declare local arrays.
     * They are allocated from stack space and may result
     * in hard to find stack overflows. However, 16 bytes
     * should be acceptable.
     */
    char buffer[16];

    /*
     * Register the debug output device (UART0 by default), open it, 
     * assign stdout to it and set the baudrate.
     */
    NutRegisterDevice(&DEV_DEBUG, 0, 0);
    freopen(DEV_DEBUG_NAME, "w", stdout);
    _ioctl(_fileno(stdout), UART_SETSPEED, &baud_dbg);

    /*
     * Print a banner with version information on the debug output.
     */
    printf("\nNut/OS %s - RS485 Slave Test\n", NutVersionString());

    /*
     * Register UART1 device. If this fails, we give up.
     */
    if (NutRegisterDevice(&DEV_UART1, 0, 0)) {
        printf("Failed to register %s\n", DEV_UART1_NAME);
        /* Freeze the system. */
        for(;;);
    }

    /*
     * Open a file stream on the UART1 device for reading and writing in 
     * cooked mode. With Nut/OS 4.2.1 and earlier we must open the stream 
     * in binary mode. Again, we give up on failures.
     */
    if ((rs485 = fopen(DEV_UART1_NAME, "r+")) == NULL) {
        printf("Failed to open %s\n", DEV_UART1_NAME);
        /* Freeze the system. */
        for(;;);
    }

    /*
     * For RS485 set flow control to half duplex mode. Failures are 
     * ignored.
     */
    if (_ioctl(_fileno(rs485), UART_SETFLOWCONTROL, &flow_485)) {
        puts("Failed to set flow control");
    }

    /*
     * Set RS485 baud rate. Failures are ignored.
     */
    if (_ioctl(_fileno(rs485), UART_SETSPEED, &baud_485)) {
        puts("Failed to set baudrate");
    }

    /*
     * Set RS485 read time out. Failures are ignored.
     */
    if (_ioctl(_fileno(rs485), UART_SETREADTIMEOUT, &to_485)) {
        puts("Failed to set read timeout");
    }

    /*
     * Enable the pull-up on the receiver line (PD2). Otherwise the line 
     * will be floating during transmission and will confuse the UART
     * receiver.
     */
    sbi(PORTD, 2);

    /*
     * The main application loop echos any incoming line.
     */
    for (;;) {
        /* Read the next line from UART1. */
        if (fgets(buffer, sizeof(buffer), rs485)) {
            printf("Rx(%d):%s", strlen(buffer), buffer);
            /*
             * fgets() will return on the first incoming carriage return
             * or linefeed character. If the remote is sending carriage
             * return plus linefeed, then the linefeed transmission may
             * be still active. Thus, we add a one character delay before
             * sending back our response. 10 ms is sufficient at 1200 Baud.
             */
            NutSleep(10);
            /* Send the received line back to the remote. */
            fputs(buffer, rs485);
            /*
             * The UART driver will not immediately start transmission,
             * but add the characters to the internal transmit buffer.
             * A flush is required to start the transfer.
             */
            fflush(rs485);
            printf("Tx(%d):%s", strlen(buffer), buffer);
        }

        /*
         * fgets() returned a NULL pointer. Check, if this was caused by
         * a time out or an error.
         */
        else if (ferror(rs485)) {
            puts("Receive error!");
        }
        else {
            puts("Receive time out!");
        }
    }
}
